#include "SampleExam1/SampleExam1.hpp"

using namespace cfl;

class HistSpot: public IResetValues
{
public:
  HistSpot(const cfl::AssetModel & rModel) 
    :m_rModel(rModel)
  {};
		
  Slice resetValues(unsigned iTime, double dBeforeReset) const 
  {
    return m_rModel.spot(iTime);
  }

private:
  const cfl::AssetModel & m_rModel;
};

cfl::PathDependent 
histSpot(double dInitialValue, const std::vector<unsigned> & rResetIndexes, 
	 const AssetModel & rModel) 
{
  return PathDependent(new HistSpot(rModel), rResetIndexes, 
		       dInitialValue); 
}


cfl::MultiFunction prb::
clique(double dMaturity, const std::vector<double> & rAverageTimes, 
       const std::vector<double> & rResetTimes, double dInitialStrike,  
       AssetModel & rModel)
{
  PRECONDITION(rAverageTimes.front()>rModel.initialTime()); 
  PRECONDITION(rAverageTimes.back() < dMaturity);

  std::vector<double> uEventTimes(rResetTimes.size()+rAverageTimes.size()+1);
  uEventTimes.front() = rModel.initialTime();
  //add payment times and reset times
  std::vector<double>::iterator itEnd = 
    std::set_union(rResetTimes.begin(),rResetTimes.end(),
		   rAverageTimes.begin(),rAverageTimes.end(),
		   uEventTimes.begin()+1);
  uEventTimes.resize(itEnd-uEventTimes.begin());
  rModel.assignEventTimes(uEventTimes);
  
  std::vector<unsigned> uResetIndexes(rResetTimes.size());
  for (unsigned iI=0; iI<uResetIndexes.size(); iI++) {
    uResetIndexes[iI] = 
      std::lower_bound(uEventTimes.begin(), uEventTimes.end(), 
		       rResetTimes[iI]) - uEventTimes.begin();
  }
  unsigned iStrike = 
    rModel.addState(histSpot(dInitialStrike,uResetIndexes, rModel));

  int iTime = rModel.eventTimes().size()-1;
  Slice uOption = rModel.cash(iTime, 0.);
  while (iTime > 0) {
    //uOption is the present value of all future call payoffs in the sum 
    if (std::binary_search(rAverageTimes.begin(), rAverageTimes.end(), 
			   rModel.eventTimes()[iTime])) {
      uOption += max(rModel.spot(iTime)-rModel.state(iTime, iStrike),0.)
	*rModel.discount(iTime, dMaturity);  
    }
    iTime--;
    uOption.rollback(iTime);
  }
  //not forgetting to divide on number of averaging times
  uOption /= rAverageTimes.size();

  return interpolate(uOption, iStrike);
}
