#include "SampleExam1/SampleExam1.hpp"

using namespace cfl;

cfl::MultiFunction 
prb::upRangeOutPut(double dUpperBarrier, unsigned iOutTimes, 
		   const std::vector<double> & rBarrierTimes,
		   double dStrike, double dMaturity, 
		   AssetModel & rModel)
{
  PRECONDITION(rModel.initialTime() < rBarrierTimes.front());
  PRECONDITION(rBarrierTimes.back() < dMaturity);
  PRECONDITION(iOutTimes > 0);
  PRECONDITION(iOutTimes <= rBarrierTimes.size());

  std::vector<double> uEventTimes(1, rModel.initialTime());
  uEventTimes.insert(uEventTimes.end(), rBarrierTimes.begin(), 
		     rBarrierTimes.end());
  uEventTimes.insert(uEventTimes.end(), dMaturity);
  rModel.assignEventTimes(uEventTimes);

  int iTime = uEventTimes.size()-1;
  Slice uPut = max(dStrike - rModel.spot(iTime), 0.);
  std::vector<Slice> uOption(iOutTimes, uPut);
  iTime--;
  for (unsigned iI=0;iI<uOption.size();iI++) {
    uOption[iI].rollback(iTime);
  } 
  	
  while (iTime > 0) {
    //uOption[iI]: the value under the condition that exactly 
    //iI barrier events took place before and at iTime. 
    Slice uInd = indicator(rModel.spot(iTime),dUpperBarrier);
    for (unsigned iI=0; iI+1<iOutTimes; iI++) {
      uOption[iI] += uInd*(uOption[iI+1]-uOption[iI]);
    }
    uOption.back()*=(1.-uInd);
    iTime--;
    for (unsigned iI=0;iI<uOption.size();iI++) {
      uOption[iI].rollback(iTime);
    }
  }
  return interpolate(uOption.front());	
}
