#include "SampleExam1/SampleExam1.hpp"

using namespace cfl;

class ForwardStockDividends: public IFunction
{
public:
  ForwardStockDividends(double dSpot, const std::vector<double> & rTimes, 
			const std::vector<double> & rDividends, 
			const Function & rDiscount, 
			double dInitialTime)
    :m_uDiscount(rDiscount), m_uDividends(rDividends), m_uTimes(rTimes)
  {
    PRECONDITION(rTimes.front() > dInitialTime);
    PRECONDITION(rTimes.size() == rDividends.size());
    m_dSpot = dSpot;
    m_dInitialTime = dInitialTime;
  }

  bool belongs(double dT) const 
  { 
    return (dT >= m_dInitialTime) && (m_uDiscount.belongs(dT)) 
      && (dT <= m_uTimes.back());
  }

  double operator()(double dTime) const
  {
    PRECONDITION(belongs(dTime));			
	
    unsigned iTime = 0;
    double dPresentValue = m_dSpot;			
    while((dTime > m_uTimes[iTime])&&(iTime < m_uTimes.size())) {
      dPresentValue -= (m_uDividends[iTime]) * m_uDiscount(m_uTimes[iTime]);
      iTime++;
    }			
    double dForward = dPresentValue / m_uDiscount(dTime);		
    return dForward;			
  }
		
private:
  Function m_uDiscount;
  std::vector<double> m_uDividends, m_uTimes;
  double m_dSpot, m_dInitialTime;
};

cfl::Function prb::
forwardStockDividends(double dSpot, 
		      std::vector<double> & rFixedDividendsTimes, 
		      std::vector<double> & rFixedDividends, 
		      const cfl::Function & rDiscount, 
		      double dInitialTime)
{
  return Function(new ForwardStockDividends(dSpot, rFixedDividendsTimes, 
					    rFixedDividends, rDiscount, dInitialTime));
}
