#include "SampleExam2/SampleExam2.hpp" 

using namespace cfl;

cfl::MultiFunction 
prb::boost(double dNotional, double dLowerBarrier, 
	   double dUpperBarrier,  
	   const std::vector<double> & rBarrierTimes,
	   AssetModel & rModel)
{
  PRECONDITION(rBarrierTimes.front() > rModel.initialTime());
  PRECONDITION(dLowerBarrier < dUpperBarrier);

  std::vector<double> uEventTimes(rBarrierTimes);
  uEventTimes.insert(uEventTimes.begin(), rModel.initialTime());
  rModel.assignEventTimes(uEventTimes);
  
  unsigned iTime = rModel.eventTimes().size()-1;
  ASSERT(iTime == rBarrierTimes.size());
  Slice uOption = rModel.cash(iTime, dNotional);
  while (iTime > 0) {
    //uOption is the value to continue
    //payoff if stop today
    double dPayoff = dNotional*(iTime-1.)/rBarrierTimes.size();
    Slice uIndStop = indicator(dLowerBarrier, rModel.spot(iTime)) + 
      indicator(rModel.spot(iTime), dUpperBarrier);
    uOption += (dPayoff - uOption)*uIndStop;
    iTime--;
    uOption.rollback(iTime);
  }
  return interpolate(uOption);
}
