#include "cfl/Interp.hpp"
#include "SampleExam2/SampleExam2.hpp" 

using namespace cfl;


cfl::Function 
prb::discountSwapLogLinearInterp(const std::vector<double> & rSwapRates, 
				 double dPeriod, double dInitialTime)
{
  PRECONDITION(rSwapRates.size() > 0); 

  //times for interpolation = initial time + payment times
  std::vector<double> uTimes(rSwapRates.size()+1);
  uTimes.front() = dInitialTime; 
  std::transform(uTimes.begin(), uTimes.end()-1, uTimes.begin()+1, 
		 [dPeriod](double dX){ return dX+dPeriod; }); 

  std::vector<double> uDiscount(uTimes.size());
  uDiscount.front() = 1.;
  double dSum = 0;
  for (unsigned iI=1; iI<uDiscount.size(); iI++) {
    uDiscount[iI] = 
      (1 - dSum*rSwapRates[iI-1]*dPeriod)/(1 + rSwapRates[iI-1]*dPeriod);
    dSum += uDiscount[iI];
  }
  std::vector<double> uLogDiscount(uDiscount.size());
  std::transform(uDiscount.begin(), uDiscount.end(), uLogDiscount.begin(), 
		 [](double dX){ return std::log(dX); }); 

  //linear interpolation of the logarithm of discount factors
  cfl::Interp uLinear = NInterp::linear(); 
  Function uLogDiscountFunction = 
    uLinear.interpolate(uTimes.begin(), uTimes.end(), uLogDiscount.begin()); 

  return cfl::exp(uLogDiscountFunction);
}
