#include <iostream>
#include <fstream>
#include "test/Main.hpp"
#include "test/Black.hpp"
#include "test/HullWhite.hpp"
#include "test/Data.hpp"
#include "SampleExam2/test_SampleExam2.hpp"
#include "SampleExam2/SampleExam2.hpp"

using namespace std;
using namespace prb;
using namespace cfl;
using namespace prb::test;
using namespace cfl::test;
using namespace cfl::test::Black;
using namespace cfl::test::HullWhite;

namespace test_prb
{
  static const char * c_pDataFile = 
    "../SampleExam2/Output/SampleExam2Data.txt";
  static const char * c_pDataProject = "TEST OF DATA CURVES";
  static const char * c_pBlackStandardFile = 
    "../SampleExam2/Output/SampleExam2BlackStandard.txt";
  static const char * c_pBlackStandardProject = 
    "TEST OF STANDARD OPTIONS IN BLACK MODEL FOR SINGLE ASSET";
  static const char * c_pHullWhiteStandardFile = 
    "../SampleExam2/Output/SampleExam2HullWhiteStandard.txt";
  static const char * c_pHullWhiteStandardProject = 
    "TEST OF STANDARD OPTIONS IN HULL AND WHITE  MODEL FOR INTEREST RATES";
  static const char * c_pHullWhitePathDependentFile = 
    "../SampleExam2/Output/SampleExam2HullWhitePathDependent.txt";
  static const char * c_pHullWhitePathDependentProject = 
    "TEST OF PATH DEPENDENT OPTIONS IN HULL AND WHITE MODEL FOR INTEREST RATES";

  void data();
  void standard(const cfl::Black::Data & rData);
  void standard(const cfl::HullWhite::Data & rData);
  void pathDependent(const cfl::HullWhite::Data & rData);
}

int main()
{
  using namespace cfl::test;
  using namespace std;
  try{
    cfl::test::Data::project(test_prb::data, test_prb::c_pDataProject, 
			     test_prb::c_pDataFile);
    cfl::test::pause();
    cfl::test::Black::project(test_prb::standard, 
			      test_prb::c_pBlackStandardProject, 
			      test_prb::c_pBlackStandardFile);
    cfl::test::pause();
    cfl::test::HullWhite::project(test_prb::standard, 
				  test_prb::c_pHullWhiteStandardProject, 
				  test_prb::c_pHullWhiteStandardFile);
    cfl::test::pause();
    cfl::test::HullWhite::project(test_prb::pathDependent, 
				  test_prb::c_pHullWhitePathDependentProject, 
				  test_prb::c_pHullWhitePathDependentFile);
    cfl::test::pause();
  }
  catch(const std::exception & rError) {
    std::cerr<< rError.what() << endl;
    cfl::test::pause();
  }
}

void test_prb::data()
{
  prb::test::discountSwapLogLinearInterp();
}

void test_prb::standard(const cfl::Black::Data & rData)
{
  double dQuality = cfl::test::Black::c_dQuality;
  cout << "quality = " << dQuality << endl;
  double dSpot = rData.forward()(rData.initialTime());
  double dInterval = cfl::test::Black::c_dInterval;
  unsigned iPoints = cfl::test::Black::c_iPoints;

  AssetModel uModel = cfl::Black::model(rData, dInterval, dQuality);

  using namespace prb::test; 
  
  report(prb::test::boost, dSpot, dInterval, iPoints, uModel);
}

void test_prb::standard(const cfl::HullWhite::Data & rData)
{
  double dQuality = cfl::test::HullWhite::c_dQuality;
  cout << "quality = " << dQuality << endl;
  const double c_dDay = 1./365.;
  double dRate = 
    - std::log(rData.discount()(rData.initialTime()+c_dDay))/c_dDay;
  double dInterval = cfl::test::HullWhite::c_dInterval;
  unsigned iPoints = cfl::test::HullWhite::c_iPoints;

  InterestRateModel uModel = cfl::HullWhite::model(rData, dInterval, dQuality);

  using namespace prb::test;

  report(resetCouponPutBond, dRate, dInterval, iPoints, uModel);
}

void test_prb::pathDependent(const cfl::HullWhite::Data & rData)
{
  double dQuality = cfl::test::HullWhite::c_dQuality;
  double dPathDependQuality =
    cfl::test::HullWhite::c_dPathDependQuality;
  
  cout << "quality = " << dQuality << endl;
  cout << "quality for additional state processes = " <<
    dPathDependQuality << endl;
  const double c_dDay = 1./365.;
  double dRate = 
    - std::log(rData.discount()(rData.initialTime()+c_dDay))/c_dDay;
  double dInterval = cfl::test::HullWhite::c_dInterval;
  unsigned iPoints = cfl::test::HullWhite::c_iPoints;

  InterestRateModel uModel =
    cfl::HullWhite::model(rData, dInterval, dQuality, dPathDependQuality);

  using namespace prb::test;

  report(resetCap, dRate, dInterval, iPoints, uModel);
}

void prb::test::discountSwapLogLinearInterp() 
{
  cfl::test::print("DISCOUNT CURVE OBTAINED FROM SWAP RATES BY LOG LINEAR INTERPOLATION");
  cout << endl;
  double dRate = 0.12;
  double dRateStep = 0.005;
  double dInitialTime = 1.;
  unsigned iSwaps = 10;
  std::vector<double> uSwapRates(iSwaps);
  double dPeriod = 0.5;
  uSwapRates.front() = dRate;
  std::transform(uSwapRates.begin(), uSwapRates.end()-1, uSwapRates.begin()+1, 
		 [dRateStep](double dX){ return dX+dRateStep; }); 
  cout << "initial time = " << dInitialTime << endl;
  cout << "period between payments = " << dPeriod << endl;
  cout << "swap rates: " << endl;
  cfl::test::print(uSwapRates.begin(), uSwapRates.end());
  Function uDiscount = 
    prb::discountSwapLogLinearInterp(uSwapRates, dPeriod, dInitialTime); 
  double dInterval = iSwaps*dPeriod/1.1;
  cfl::test::Data::print(uDiscount, dInitialTime + 0.001, dInterval); 	
}

MultiFunction prb::test::boost(AssetModel & rModel) 
{
  cfl::test::print("BOOST OPTION IN ASSET MODEL");
  cout << endl;
  double dNotional = cfl::test::Black::c_dNotional;
  double dLowerBarrier = cfl::test::Black::c_dSpot*0.95;
  double dUpperBarrier = cfl::test::Black::c_dSpot*1.1;
  const std::vector<double> uBarrierTimes = cfl::test::Black::c_uExerciseTimes;
  cout << "notional = " << dNotional << endl;
  cout << "lower barrier = " << dLowerBarrier << endl;
  cout << "upper barrier = " << dUpperBarrier << endl;
  cout << "barrier times = "<< endl;
  cfl::test::print(uBarrierTimes.begin(), uBarrierTimes.end());
  return prb::boost(dNotional, dLowerBarrier, dUpperBarrier,
		    uBarrierTimes, rModel);
}

cfl::MultiFunction prb::test::resetCouponPutBond(InterestRateModel & rModel) 
{
  cfl::test::print("PUTABLE BOND WITH RESETTABLE COUPON IN INTEREST RATE MODEL");
  cout << endl;
  cfl::Data::CashFlow uBondParameters = cfl::test::HullWhite::swapParameters();
  double dResetCouponRate = uBondParameters.rate - 0.02;
  double dRedemptionPrice = 0.98;
  cout << "bond parameters:" << endl;
  cfl::test::print(uBondParameters);
  cout << "reset value for coupon rate = " << dResetCouponRate << endl;
  cout << "redemption price (percent of notional) = " 
       << dRedemptionPrice << endl;
  return prb::resetCouponPutBond(uBondParameters, dResetCouponRate, 
				 dRedemptionPrice, rModel);
}

cfl::MultiFunction prb::test::
resetCap(InterestRateModel & rModel) 
{
  cfl::test::print("RESETTABLE INTEREST RATE CAP IN HULL-WHITE MODEL");
  cout << endl;
  cfl::Data::CashFlow uCapParameters = cfl::test::HullWhite::swapParameters();
  double dResetMaturity = 
    uCapParameters.period*(uCapParameters.numberOfPayments-1)/1.1;
  unsigned iResetTimes = uCapParameters.numberOfPayments/3;
  std::vector<double> uResetTimes = 
    cfl::test::times(rModel.initialTime(), dResetMaturity, iResetTimes);
  double dSpread = 0.01;
  cout << "reset times = " << endl;
  cfl::test::print(uResetTimes.begin(), uResetTimes.end());
  cout << "cap parameters:" << endl;
  cfl::test::print(uCapParameters);
  cout << "spread = " << dSpread << endl;
  return  prb::resetCap(uCapParameters, dSpread, uResetTimes, rModel);
}
