#include "SampleExam3/SampleExam3.hpp"

using namespace cfl;

cfl::MultiFunction prb::
compoundCallAmericanPut(double dMaturity, double dCompoundCallStrike, 
			double dPutStrike, 
			const std::vector<double> & rExerciseTimes, 
			AssetModel & rModel)
{
  PRECONDITION(rModel.initialTime() < dMaturity);
  PRECONDITION(dMaturity < rExerciseTimes.front());

  std::vector<double> uEventTimes(rExerciseTimes);
  uEventTimes.insert(uEventTimes.begin(), dMaturity);
  uEventTimes.insert(uEventTimes.begin(), rModel.initialTime());
  rModel.assignEventTimes(uEventTimes);
  
  int iTime = uEventTimes.size()-1;
  Slice uPut = rModel.cash(iTime, 0.);
  while (iTime > 1) {
    uPut = max(uPut, dPutStrike - rModel.spot(iTime));
    iTime--;
    uPut.rollback(iTime);
  }
  ASSERT(uEventTimes[iTime] == dMaturity);
  Slice uCall = max(uPut - dCompoundCallStrike, 0.);
  uCall.rollback(0);
  return interpolate(uCall);	
}
