#include <numeric>
#include "SampleExam3/SampleExam3.hpp"
#include "cfl/Interp.hpp"

using namespace cfl;
using namespace cfl::Data;

class VolatilityFromVariance: public cfl::IFunction
{
public:
  VolatilityFromVariance(const cfl::Function & rVariance, double dInitialTime)
    :m_uVariance(rVariance), m_dInitialTime(dInitialTime)
  {}

  double operator()(double dT) const 
  {
    PRECONDITION(belongs(dT));
    double dVolatility;
    double dTime = std::max(dT, m_dInitialTime + cfl::c_dEps); 
    dVolatility = std::sqrt(m_uVariance(dTime)/(dTime - m_dInitialTime));
    return dVolatility;
  }

  bool belongs(double dT) const 
  {
    return m_uVariance.belongs(dT);
  }

private:
  cfl::Function m_uVariance;
  double m_dInitialTime;
};

cfl::Function prb::
volatilityLinearInterpOfVar(const std::vector<double> & rTimes, 
			    const std::vector<double> & rVolatilities, 	
			    double dInitialTime)
{
  PRECONDITION(rTimes.size() == rVolatilities.size()); 
  PRECONDITION(rTimes.size()>0);
  PRECONDITION(rTimes.front()>dInitialTime);

  std::vector<double> uVariance(rVolatilities.size()+1,0.);
  std::transform(rTimes.begin(), rTimes.end(), rVolatilities.begin(), 
		 uVariance.begin()+1, [dInitialTime](double dT, double dVol) {
		   return dVol*dVol*(dT-dInitialTime); }); 
  std::vector<double> uTimes(rTimes);
  uTimes.insert(uTimes.begin(), dInitialTime);
  Interp uLinear = cfl::NInterp::linear();
  Function uVar = 
    uLinear.interpolate(uTimes.begin(),uTimes.end(),uVariance.begin());

  return cfl::Function(new VolatilityFromVariance(uVar, dInitialTime));
}
