#include "SampleExam3/SampleExam3.hpp"

using namespace cfl;

cfl::MultiFunction prb::
callableCappedFloater(const Data::CashFlow & rCap, double dLiborSpread, 
		      InterestRateModel & rModel)
{
  std::vector<double> uEventTimes(rCap.numberOfPayments);
  uEventTimes.front() = rModel.initialTime();
  std::transform(uEventTimes.begin(), uEventTimes.end()-1, 
		 uEventTimes.begin()+1, 
		 [&rCap](double dX){ return dX+rCap.period; }); 
  rModel.assignEventTimes(uEventTimes); 

  //last minus one payment time
  int iTime = rModel.eventTimes().size() - 1; 
  Slice uDiscount = 
    rModel.discount(iTime, rModel.eventTimes()[iTime] + rCap.period);
  double dSpreadFactor = rCap.notional*(dLiborSpread*rCap.period - 1.); 
  double dCapFactor = rCap.notional*rCap.rate*rCap.period;
  Slice uValueOfNextCoupon = 
    min(rCap.notional + dSpreadFactor*uDiscount, dCapFactor*uDiscount);
  //uOption is the value to continue (includes next coupon)
  Slice uOption = rCap.notional*uDiscount + uValueOfNextCoupon; 
  while (iTime >0) {
    //uOption is the value to continue (includes the value of the next coupon).
    uOption = min(uOption, rCap.notional);
    iTime--;
    uOption.rollback(iTime);
    uDiscount = 
      rModel.discount(iTime, rModel.eventTimes()[iTime] + rCap.period);
    uValueOfNextCoupon =
      min(rCap.notional + dSpreadFactor*uDiscount, dCapFactor*uDiscount);
    uOption += uValueOfNextCoupon;
  }
  return interpolate(uOption);
}
